// koboshi.c by Kosei Demura 2007-1-20
// My web site is http://demura.net
// This program uses the Open Dynamics Engine (ODE) by Russell Smith.
// The ODE web site is http://ode.org/
#include <ode/ode.h>
#include <drawstuff/drawstuff.h>

#ifdef dDOUBLE
#define dsDrawSphere dsDrawSphereD
#endif

#define MAX_CONTACTS 4

static dWorldID world;
static dSpaceID space; 
static dGeomID  ground;
static dJointGroupID contactgroup; 
dsFunctions fn;

typedef struct {    
  dBodyID body;     
  dGeomID geom;     
  dReal   radius;
  dReal   length;
  dReal   mass;
} MyLink;

MyLink   head, torso; // ܂̓A 
dJointID joint;

static void nearCallback (void *data, dGeomID o1, dGeomID o2)
{
  int i;

  // exit without doing anything if the two bodies are connected by a joint             
  dBodyID b1 = dGeomGetBody(o1);
  dBodyID b2 = dGeomGetBody(o2);
  if (b1 && b2 && dAreConnectedExcluding (b1,b2,dJointTypeContact)) return;

  dContact contact[MAX_CONTACTS];   // up to MAX_CONTACTS contacts per box-box          
  for (i=0; i<MAX_CONTACTS; i++) {
    contact[i].surface.mode = dContactBounce | dContactSoftCFM;
    contact[i].surface.mu = 1.0; //dInfinity;
    //contact[i].surface.mu2 = dInfinity;
    contact[i].surface.bounce = 0.01;
    contact[i].surface.bounce_vel = 0.01;
    contact[i].surface.soft_cfm = 0.00001;
  }
  if (int numc = dCollide (o1,o2,MAX_CONTACTS,&contact[0].geom,
                           sizeof(dContact))) {
    for (i=0; i<numc; i++) {
      dJointID c = dJointCreateContact (world,contactgroup,contact+i);
      dJointAttach (c,b1,b2);
    }
  }
}

static void drawSphere(dGeomID g)
{
  if (!g) return;
  const dReal *pos = dGeomGetPosition(g);
  const dReal *R = dGeomGetRotation(g);

  dsDrawSphere(pos,R,dGeomSphereGetRadius(g));
}

static void simLoop (int pause)
{
  dSpaceCollide(space,0,&nearCallback); 
  dWorldStep(world,0.01);
  dJointGroupEmpty(contactgroup);

  dsSetColor(1.0,0.0,0.0);
  drawSphere(torso.geom);
  dsSetColor(1.0, 0.0, 0.0);
  drawSphere(head.geom);
}

void start()
{
  static float xyz[3] = {   3.5,0.0,1.0};
  static float hpr[3] = {-180.0,0.0,0.0};
  dsSetSphereQuality(3);
  dsSetViewpoint(xyz,hpr);
}

// Create an object 
void createDaruma() 
{
  dMass m,m1;
  dReal x0 = 0.0, y0 = 0.0, z0 = 0.4;
  dReal dx = 0.0, dy = 0.0, dz = 0.4;  // ItZbg
 
  // ́FdSʒuړĐݒ
  torso.body   = dBodyCreate(world);  
  torso.radius = 0.4;
  torso.mass   = 10.0;

  dMassSetZero(&m);
  dMassSetSphereTotal(&m,torso.mass,torso.radius);
  dBodySetMass(torso.body, &m);
  dBodySetMass(torso.body,&m); 

  torso.geom = dCreateSphere(space,torso.radius);   
  dGeomSetBody(torso.geom,torso.body);   
  dBodySetPosition(torso.body, x0, y0, z0 - dz);
  dGeomSetOffsetPosition(torso.geom, dx, dy,dz);
  
  // 
  head.body = dBodyCreate(world);
  head.radius = 0.3;
  head.mass   = 0.001;
 
  dMassSetZero(&m1);
  dMassSetSphereTotal(&m1,head.mass, head.radius);
  dBodySetMass(head.body,&m1);

  head.geom = dCreateSphere(space,head.radius); 
  dGeomSetBody(head.geom,head.body);                
  dGeomSetPosition(head.geom, x0, y0, z0 + torso.radius + head.radius);
  
  // fixed joint
  joint = dJointCreateFixed(world, 0);
  dJointAttach(joint, torso.body,head.body);
  dJointSetFixed(joint); 
}


void command(int cmd)
{
  switch(cmd) {
  case 'f':
    dBodyAddTorque(head.body,  100, 0, 0); break;
  case 'j':
    dBodyAddTorque(head.body, -100, 0, 0); break;
  }
}

void  setDrawStuff() {
  fn.version = DS_VERSION;
  fn.start   = &start;
  fn.step    = &simLoop;
  fn.command = &command;
  fn.stop    = NULL;
  fn.path_to_textures = "../../drawstuff/textures";
}

int main(int argc, char **argv)
{
  setDrawStuff();

  world = dWorldCreate();
  space = dHashSpaceCreate(0);
  contactgroup = dJointGroupCreate(0); 

  dWorldSetGravity(world,0,0,-9.8);
  ground = dCreatePlane(space,0,0,1,0);
  createDaruma();

  dsSimulationLoop (argc,argv,640,480,&fn);

  dWorldDestroy (world);
  
  return 0;
}
